#include <MDR32F9Qx_port.h>
#include <MDR32F9Qx_rst_clk.h>
#include <MDR32F9Qx_eeprom.h>
#include <MDR32F9Qx_uart.h>

void Delay(int waitTicks);

#define VD3 PORT_Pin_0
#define VD4 PORT_Pin_1
#define LED_ALL VD3 | VD4

#define CYCLES_IN_LOOP                     (8UL)
#define GET_LOOPS_IN_N_US(N)               ((uint32_t)( (float)(N) * 80.0 / CYCLES_IN_LOOP) ) // N * кол-во циклов за 1 мкс 
#define GET_LOOPS_FOR_TRANSACTION(SPEED)   ((uint32_t)( GET_LOOPS_IN_N_US( ((float) 1e6 / SPEED) * 10 ) )) // кол-во мкс для передачи одного бита * кол-во бит в посылке = кол-во мкс для одной посылки

#define RESET_FLAG_OE

void CPU_Initialize (void);
void UART_Initialize(void);
void UART_SoftReset(MDR_UART_TypeDef* UARTx, uint32_t speed_uart);
void ResetFlagOE(MDR_UART_TypeDef* UARTx);

int count, n, tmp[100];

int main()
{
    PORT_InitTypeDef GPIOInitStruct;

    CPU_Initialize();
    UART_Initialize();

    RST_CLK_PCLKcmd (RST_CLK_PCLK_PORTC | RST_CLK_PCLK_DMA, ENABLE);
    PORT_StructInit(&GPIOInitStruct);

    GPIOInitStruct.PORT_Pin        = LED_ALL;
    GPIOInitStruct.PORT_OE         = PORT_OE_OUT;
    GPIOInitStruct.PORT_SPEED      = PORT_SPEED_SLOW;
    GPIOInitStruct.PORT_PULL_UP    = PORT_PULL_UP_ON;
    GPIOInitStruct.PORT_MODE       = PORT_MODE_DIGITAL;
    PORT_Init(MDR_PORTC, &GPIOInitStruct);

    while (1)
    {
        count = 1;
        
        while (UART_GetFlagStatus (MDR_UART2, UART_FLAG_TXFF) != SET) // Заполнение буфера FIFO_TX
            UART_SendData (MDR_UART2, count++);
        
        while (UART_GetFlagStatus (MDR_UART2, UART_FLAG_BUSY) == SET); // Ожидание завершения отправки (буфер приёмника полон)
        
        while (UART_GetFlagStatus (MDR_UART2, UART_FLAG_TXFF) != SET) // Заполнение буфера FIFO_TX
            UART_SendData (MDR_UART2, count++);
        
        UART_SoftReset(MDR_UART2, 115200); // Программный сброс UART
        
        UART_Initialize();                 // Инициализация UART
        
        count = 10;

        for(n = 0; n < 16; n++){
            UART_SendData (MDR_UART2, count++);
            while(UART_GetFlagStatus(MDR_UART2, UART_FLAG_BUSY) == SET);
            tmp[count] = UART_ReceiveData(MDR_UART2);
            __NOP();
        }
        
        PORT_SetBits(MDR_PORTC, LED_ALL);	// LED ON
        Delay( GET_LOOPS_FOR_TRANSACTION(115200) );
        
        PORT_ResetBits(MDR_PORTC, LED_ALL); // LED OFF
        Delay( GET_LOOPS_FOR_TRANSACTION(115200) );
    }      
}

void UART_SoftReset(MDR_UART_TypeDef* UARTx, uint32_t speed_uart) // При переполнении может остаться флаг OE (28.5.7 Бит переполнения буфера)
{   
    UARTx->IMSC = 0x00;  // Запрещение прерываний
    UARTx->DMACR = 0x00; // Запрещение запросов DMA
    
    UARTx->CR &= ~(1 << 0); // Запрещение работы приёмопередатчика UART, при этом перед остановкой приёмопередатчик UART завершает приём/передачу текущего символа 
    Delay( GET_LOOPS_FOR_TRANSACTION(speed_uart) ); // Ожидание завершения последней транзакции
   
    #ifdef RESET_FLAG_OE
        ResetFlagOE(MDR_UART2);
    #else
        if ( !(UARTx->LCR_H & UART_FIFO_ON) )  // Если FIFO выключено
            UARTx->LCR_H = UART_FIFO_ON;       // Включить FIFO, чтобы сбросить буфер приёмника и передатчика
    #endif
    
    UARTx->IBRD = 0x00;
    UARTx->FBRD = 0x00;
    UARTx->LCR_H = 0x00;   // Сброс FIFO передатчика и приёмника
    
    UARTx->RSR_ECR = 0x00; // Сброс ошибок RSR_ECR
    UARTx->ICR = 0xFFFF;   // Сброс запросов прерываний ICR
    
    UARTx->ILPR = 0x00;    // Установка оставшихся регистров значениями по сбросу
    UARTx->CR = 0x0300;
    UARTx->IFLS = 0x12;
}

void ResetFlagOE(MDR_UART_TypeDef* UARTx)
{
    volatile int temp = 0;
    
    MDR_PORTF->ANALOG &= ~( (1<<1) | (1<<0)); // Отключение выводов UART RX и TX
    
    if ( !(UARTx->LCR_H & UART_FIFO_ON) )  // Если FIFO выключено
        UARTx->LCR_H |= UART_FIFO_ON;      // Включить FIFO, чтобы сбросить буфер приёмника и передатчика
    else
        UARTx->LCR_H &= ~UART_FIFO_ON;     // Выключить FIFO, чтобы сбросить FIFO приёмника и передатчика
    
    UARTx->CR |= (1<<7) | (1<<0); // Разрешение работы в режиме шлейфа
    
    UARTx->DR = 0x00; // Отправка, чтобы после приёма при чтении регистра DR бит OE установился, сообщая о том, что ранее было переполнение и потеря принимаемых символов
    while(UARTx->FR & UART_FLAG_BUSY){} // Ожидание окончания приёма
    temp = UARTx->DR;
    
    UARTx->CR &= ~(1 << 0); // Запрещение работы приёмопередатчика UART
    
    MDR_PORTF->ANALOG |= ( (1<<1) | (1<<0)); // Включение выводов RX и TX
}

void UART_Initialize(void)
{
    UART_InitTypeDef UART_InitStructure;
    PORT_InitTypeDef PortInit;
    
    RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTF,ENABLE);
    
    PORT_StructInit(&PortInit);
    
    PortInit.PORT_Pin = PORT_Pin_0 | PORT_Pin_1;
    PortInit.PORT_MODE = PORT_MODE_DIGITAL;
    PortInit.PORT_SPEED = PORT_SPEED_FAST;
    PortInit.PORT_FUNC = PORT_FUNC_OVERRID;
    
    PORT_Init(MDR_PORTF, &PortInit);
    
    /* Enables the CPU_CLK clock on UART2 */
    RST_CLK_PCLKcmd(RST_CLK_PCLK_UART2, ENABLE);

    /* Set the HCLK division factor = 1 for UART2 */
    UART_BRGInit(MDR_UART2, UART_HCLKdiv1);
    
    UART_StructInit(&UART_InitStructure);
    
    UART_InitStructure.UART_BaudRate                = 115200;
    UART_InitStructure.UART_WordLength              = UART_WordLength8b;
    UART_InitStructure.UART_StopBits                = UART_StopBits1;
    UART_InitStructure.UART_Parity                  = UART_Parity_No;
    UART_InitStructure.UART_FIFOMode                = UART_FIFO_ON;
    UART_InitStructure.UART_HardwareFlowControl     = UART_HardwareFlowControl_RXE | UART_HardwareFlowControl_TXE;
    /* Configure UART1 parameters */
    UART_Init (MDR_UART2,&UART_InitStructure);
    
    UART_ITConfig(MDR_UART2, UART_IT_TX | UART_IT_RX | UART_IT_RT | UART_IT_OE, ENABLE);
    UART_DMACmd(MDR_UART2, UART_DMA_RXE | UART_DMA_TXE | UART_DMA_ONERR, ENABLE);
    
    /* Enables UART2 peripheral */
    UART_Cmd(MDR_UART2,ENABLE);   
}

// Инициализация системы тактирования микроконтроллера 
void CPU_Initialize(void) 
{ 
   // Сброс настроек системы тактирования 
   RST_CLK_DeInit(); 
    
   // Инициализация генератора на внешнем кварцевом резонаторе (HSE = 8 МГц) 
   RST_CLK_HSEconfig (RST_CLK_HSE_ON); 
   while (RST_CLK_HSEstatus() != SUCCESS);
    
   // Инициализация блока PLL 
   // Включение использования PLL 
   RST_CLK_CPU_PLLcmd (ENABLE); 
   // Настройка источника и коэффициента умножения PLL 
   // (CPU_C1_SEL = HSE) 
   RST_CLK_CPU_PLLconfig (RST_CLK_CPU_PLLsrcHSEdiv1, RST_CLK_CPU_PLLmul10); 
   while (RST_CLK_CPU_PLLstatus() != SUCCESS); 
   // Подключение PLL к системе тактирования 
   // (CPU_C2_SEL = PLLCPUo = 80 МГц) 
   RST_CLK_CPU_PLLuse (ENABLE);
   // Настройка коэффициента деления блока CPU_C3_SEL 
   // (CPU_C3_SEL = CPU_C2) 
   RST_CLK_CPUclkPrescaler (RST_CLK_CPUclkDIV1);
   
   // Настройка числа тактов паузы Delay в контроллере Flash-памяти
   // Тактовая частота до 100 МГц - Delay = 3
   RST_CLK_PCLKcmd (RST_CLK_PCLK_EEPROM, ENABLE);
   EEPROM_SetLatency(EEPROM_Latency_3);
   RST_CLK_PCLKcmd (RST_CLK_PCLK_EEPROM, DISABLE);

   // Настройка параметров регулятора напряжения SelectRI и LOW в контроллере BKP
   // Тактовая частота до 80 МГц
   MDR_BKP->REG_0E &= ~( 0x3F );
   MDR_BKP->REG_0E |= ( 0x6 << 3 ) | 0x6;
   
   // Использование процессором сигнала CPU_C3 
   // (HCLK = CPU_C3) 
   RST_CLK_CPUclkSelection (RST_CLK_CPUclkCPU_C3);
}


void Delay(int waitTicks)
{
  volatile uint32_t i = waitTicks;
  for (; i > 0; i--){}
}
